﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Linq.Expressions;
using Infragistics.Olap.FlatData;

namespace TestePivot
{
    public class FlatPivot : FlatDataSource
    {
        public FlatPivot(Cube cube, IEnumerable<Register> resultado)
        {
            _cube = cube;
            _structureBuilder = new StructureBuilder(cube);
            MontaEstruturaPivot();
            _l = ManipulaResultado(resultado);
            ItemsSource = _l;
        }

        private ObservableCollection<object> _l;

        private readonly Cube _cube;

        private readonly StructureBuilder _structureBuilder;

        private void MontaEstruturaPivot()
        {
            var linha = _cube.Dimensions[0];
            var coluna = _cube.Dimensions[1];

            var str = string.Format("[{0}].[{0}]", linha.PropertyName);
            Rows = GenerateInitialItems(str);
            if (coluna != null)
                Columns = GenerateInitialItems(string.Format("[{0}].[{0}]", coluna.PropertyName));

            MontaNiveis(_cube.Dimensions[0].Levels);
            MontaNiveis(_cube.Dimensions[1].Levels);
            MontaMedidas();
        }

        private void MontaMedidas()
        {
            var medidas = _cube.Measures.ToList();
            var cubeMetadata = new CubeMetadata { DataTypeFullName = _structureBuilder.GetEstrutura().FullName, DisplayName = _cube.Name };

            foreach (var medida in medidas)
            {
                var calcMeasureMetadata = new DimensionMetadata
                {
                    DisplayFormat = "{0:N2}",
                    SourcePropertyName = medida.PropertyName,
                    //DimensionType = DimensionType.Measure,
                    DisplayName = medida.Description,
                };
                cubeMetadata.DimensionSettings.Add(calcMeasureMetadata);
            }

            var str = string.Join(",", medidas.Select(x => x.PropertyName));
            Measures = GenerateInitialItems(str);

            CubesSettings.Add(cubeMetadata);
        }

        private void MontaNiveis(IList<Level> niveis)
        {
            if (niveis.Count == 0) return;

            var level = niveis.First();
            var dimensao = level.Dimension;

            var repDescriptor = new HierarchyDescriptor(dimensao.PropertyName) { SourcePropertyName = dimensao.PropertyName };
            repDescriptor.AddLevel(level.PropertyName, level.Dimension.PropertyName + "." + level.PropertyName);

            var x = Expression.Parameter(_structureBuilder.GetEstrutura(), "x");
            for (var i = 1; i < niveis.Count; i++)
            {
                level = niveis[i];
                var l = DynamicExpression.ParseLambda(new[] { x }, null, string.Format("x.{0}.Id{1}", dimensao.PropertyName, level.PropertyName));

                var pivotLevel = new HierarchyLevelDescriptor
                {
                    LevelExpressionPath = dimensao.PropertyName + "." + level.PropertyName,
                    LevelName = level.PropertyName,
                    Description = level.CompleteDescription,
                    UserDataExpression = l,
                    OrderByKeyExpression = l,
                };

                repDescriptor.LevelDescriptors.Add(pivotLevel);
            }

            AddHierarchyDescriptor(repDescriptor);
        }

        private ObservableCollection<object> ManipulaResultado(IEnumerable<Register> resultado)
        {
            var listType = typeof(ObservableCollection<>);
            var genericListType = listType.MakeGenericType(typeof(object));

            if (_l == null)
                _l = (ObservableCollection<object>)Activator.CreateInstance(genericListType);
            var list = _l;

            foreach (var resultadoItem in resultado)
            {
                var registro = ManipulaItem(resultadoItem);
                list.Add(registro);
            }

            return list;
        }

        private object ManipulaItem(Register register)
        {
            var tipoEstrutura = _structureBuilder.GetEstrutura();

            var registro = Activator.CreateInstance(tipoEstrutura);

            foreach (var medida in _cube.Measures)
            {
                if (!register.Measures.ContainsKey(medida.PropertyName)) continue;
                var valor = register.Measures[medida.PropertyName];
                var p = tipoEstrutura.GetProperty(medida.PropertyName);
                //p.SetValue(registro, valor, null);
                p.SetValue(registro, Convert.ToDouble(valor), null);
            }

            var i = 0;
            foreach (var dimension in _cube.Dimensions)
            {
                var tipoDimensao = _structureBuilder.GetEstruturaDimensao(dimension.PropertyName);
                var registroDimensao = Activator.CreateInstance(tipoDimensao);

                var levels = dimension.Levels;
                var coordenada = register.Coordenadas[i++];
                var ids = coordenada.Path.Split('\\');
                var indexId = 0;
                foreach (var level in levels)
                {
                    
                    var valorId = ids[indexId];
                    var pId = tipoDimensao.GetProperty("Id" + level.PropertyName);
                    pId.SetValue(registroDimensao, valorId, null);

                    var valor = ids[indexId];
                    var p = tipoDimensao.GetProperty(level.PropertyName);
                    p.SetValue(registroDimensao, valor, null);

                    indexId++;
                }

                registro.SetValueProperty(dimension.PropertyName, registroDimensao);
            }

            return registro;
        }
    }
}