﻿using System;
using System.Collections.Generic;
using Infragistics.Olap;

namespace TestePivot
{
    public class StructureBuilder
    {
        private readonly Type _tipoEstrutura;
        private readonly Dictionary<string, Type> _tipoEstruturaDimensoes;

        private readonly IList<DynamicTypePropertyInfo> _properties = new List<DynamicTypePropertyInfo>();

        public StructureBuilder(Cube cube)
        {
            _tipoEstruturaDimensoes = new Dictionary<string, Type>();

            var typeBuilder = new DynamicTypeBuilder
            {
                DynamicAssemblyName = "MyAssembly",
                DynamicTypeName = "Dados"
            };

            BuildMeasuresProperties(cube.Measures);
            BuildLevelsProperties(cube.Dimensions);

            _tipoEstrutura = typeBuilder.GenerateType(_properties);
        }

        private void BuildMeasuresProperties(IEnumerable<Measure> measures)
        {
            foreach (var medida in measures)
            {
                var p = new DynamicTypePropertyInfo
                {
                    PropertyName = medida.PropertyName,
                    PropertyType = typeof(double)
                };

                _properties.Add(p);
            }
        }

        private void BuildLevelsProperties(IEnumerable<Dimension> dimensions)
        {
            foreach (var dimension in dimensions)
            {
                if (_tipoEstruturaDimensoes.ContainsKey(dimension.PropertyName)) continue;
                if (_tipoEstruturaDimensoes.ContainsKey(dimension.PropertyName)) continue;

                var dimensaoProperties = new List<DynamicTypePropertyInfo>();

                var dimensaoTypeBuilder = new DynamicTypeBuilder
                {
                    DynamicAssemblyName = dimension.PropertyName + "Assembly",
                    DynamicTypeName = "Dimension" + dimension.PropertyName
                };

                foreach (var level in dimension.Levels)
                {
                    var pId = new DynamicTypePropertyInfo
                    {
                        PropertyName = "Id" + level.PropertyName,
                        PropertyType = typeof(string),
                    };
                    dimensaoProperties.Add(pId);

                    var pDescricao = new DynamicTypePropertyInfo
                    {
                        PropertyName = level.PropertyName,
                        PropertyType = typeof(string),
                    };
                    dimensaoProperties.Add(pDescricao);
                }

                _tipoEstruturaDimensoes.Add(dimension.PropertyName, dimensaoTypeBuilder.GenerateType(dimensaoProperties));

                var pDimensao = new DynamicTypePropertyInfo
                {
                    PropertyName = dimension.PropertyName,
                    PropertyType = _tipoEstruturaDimensoes[dimension.PropertyName],
                };

                _properties.Add(pDimensao);
            }
        }

        public Type GetEstrutura()
        {
            return _tipoEstrutura;
        }

        public Type GetEstruturaDimensao(string dimensao)
        {
            return _tipoEstruturaDimensoes[dimensao];
        }
    }
}